@@ -229,7 +229,9 @@ f.content = 'new content'
 f.save(branch_name='master', commit_message='Update testfile')
 
 # or for binary data
-f.content = base64.b64encode(open('image.png').read())
+# Note: decode() is required with python 3 for data serialization. You can omit
+# it with python 2
+f.content = base64.b64encode(open('image.png').read()).decode()
 f.save(branch_name='master', commit_message='Update testfile', encoding='base64')
 # end files update
 
@@ -405,3 +407,39 @@ pipeline.retry()
 # pipeline cancel
 pipeline.cancel()
 # end pipeline cancel
+
+# boards list
+boards = gl.project_boards.list(project_id=1)
+# or
+boards = project.boards.list()
+# end boards list
+
+# boards get
+board = gl.project_boards.get(board_id, project_id=1)
+# or
+board = project.boards.get(board_id)
+# end boards get
+
+# board lists list
+b_lists = board.lists.list()
+# end board lists list
+
+# board lists get
+b_list = board.lists.get(list_id)
+# end board lists get
+
+# board lists create
+# First get a ProjectLabel
+label = get_or_create_label()
+# Then use its ID to create the new board list
+b_list = board.lists.create({'label_id': label.id})
+# end board lists create
+
+# board lists update
+b_list.position = 2
+b_list.save()
+# end board lists update
+
+# board lists delete
+b_list.delete()
+# end boards lists delete
