@@ -280,13 +280,14 @@ class GitlabObject(object):
 
             raise GitlabGetError("Object not found")
 
-    def _get_object(self, k, v):
+    def _get_object(self, k, v, **kwargs):
         if self._constructorTypes and k in self._constructorTypes:
-            return globals()[self._constructorTypes[k]](self.gitlab, v)
+            return globals()[self._constructorTypes[k]](self.gitlab, v,
+                                                        **kwargs)
         else:
             return v
 
-    def _set_from_dict(self, data):
+    def _set_from_dict(self, data, **kwargs):
         if not hasattr(data, 'items'):
             return
 
@@ -294,11 +295,11 @@ class GitlabObject(object):
             if isinstance(v, list):
                 self.__dict__[k] = []
                 for i in v:
-                    self.__dict__[k].append(self._get_object(k, i))
+                    self.__dict__[k].append(self._get_object(k, i, **kwargs))
             elif v is None:
                 self.__dict__[k] = None
             else:
-                self.__dict__[k] = self._get_object(k, v)
+                self.__dict__[k] = self._get_object(k, v, **kwargs)
 
     def _create(self, **kwargs):
         if not self.canCreate:
@@ -377,27 +378,57 @@ class GitlabObject(object):
             data = self.gitlab.get(self.__class__, data, **kwargs)
             self._from_api = True
 
-        self._set_from_dict(data)
+            # the API returned a list because custom kwargs where used
+            # instead of the id to request an object. Usually parameters
+            # other than an id return ambiguous results. However in the
+            # gitlab universe iids together with a project_id are
+            # unambiguous for merge requests and issues, too.
+            # So if there is only one element we can use it as our data
+            # source.
+            if 'iid' in kwargs and isinstance(data, list):
+                if len(data) < 1:
+                    raise GitlabGetError('Not found')
+                elif len(data) == 1:
+                    data = data[0]
+                else:
+                    raise GitlabGetError('Impossible! You found multiple'
+                                         ' elements with the same iid.')
+
+        self._set_from_dict(data, **kwargs)
 
         if kwargs:
             for k, v in kwargs.items():
-                self.__dict__[k] = v
+                # Don't overwrite attributes returned by the server (#171)
+                if k not in self.__dict__ or not self.__dict__[k]:
+                    self.__dict__[k] = v
 
         # Special handling for api-objects that don't have id-number in api
         # responses. Currently only Labels and Files
         if not hasattr(self, "id"):
             self.id = None
 
-        self._set_managers()
+    def _set_manager(self, var, cls, attrs):
+        manager = cls(self.gitlab, self, attrs)
+        setattr(self, var, manager)
 
-    def _set_managers(self):
+    def __getattr__(self, name):
+        # build a manager if it doesn't exist yet
         for var, cls, attrs in self.managers:
-            manager = cls(self.gitlab, self, attrs)
-            setattr(self, var, manager)
+            if var != name:
+                continue
+            self._set_manager(var, cls, attrs)
+            return getattr(self, var)
+
+        raise AttributeError
 
     def __str__(self):
         return '%s => %s' % (type(self), str(self.__dict__))
 
+    def __repr__(self):
+        return '<%s %s:%s>' % (self.__class__.__name__,
+                               self.idAttr,
+                               getattr(self, self.idAttr))
+
     def display(self, pretty):
         if pretty:
             self.pretty_print()
@@ -550,6 +581,26 @@ class UserKeyManager(BaseManager):
     obj_cls = UserKey
 
 
+class UserProject(GitlabObject):
+    _url = '/projects/user/%(user_id)s'
+    _constructorTypes = {'owner': 'User', 'namespace': 'Group'}
+    canUpdate = False
+    canDelete = False
+    canList = False
+    canGet = False
+    requiredUrlAttrs = ['user_id']
+    requiredCreateAttrs = ['name']
+    optionalCreateAttrs = ['default_branch', 'issues_enabled', 'wall_enabled',
+                           'merge_requests_enabled', 'wiki_enabled',
+                           'snippets_enabled', 'public', 'visibility_level',
+                           'description', 'builds_enabled', 'public_builds',
+                           'import_url', 'only_allow_merge_if_build_succeeds']
+
+
+class UserProjectManager(BaseManager):
+    obj_cls = UserProject
+
+
 class User(GitlabObject):
     _url = '/users'
     shortPrintAttr = 'username'
@@ -563,10 +614,11 @@ class User(GitlabObject):
                            'projects_limit', 'extern_uid', 'provider', 'bio',
                            'admin', 'can_create_group', 'website_url',
                            'confirm', 'external']
-    managers = [
+    managers = (
         ('emails', UserEmailManager, [('user_id', 'id')]),
-        ('keys', UserKeyManager, [('user_id', 'id')])
-    ]
+        ('keys', UserKeyManager, [('user_id', 'id')]),
+        ('projects', UserProjectManager, [('user_id', 'id')]),
+    )
 
     def _data_for_gitlab(self, extra_parameters={}, update=False,
                          as_json=True):
@@ -671,10 +723,10 @@ class CurrentUser(GitlabObject):
     canUpdate = False
     canDelete = False
     shortPrintAttr = 'username'
-    managers = [
+    managers = (
         ('emails', CurrentUserEmailManager, [('user_id', 'id')]),
-        ('keys', CurrentUserKeyManager, [('user_id', 'id')])
-    ]
+        ('keys', CurrentUserKeyManager, [('user_id', 'id')]),
+    )
 
 
 class ApplicationSettings(GitlabObject):
@@ -711,6 +763,18 @@ class ApplicationSettingsManager(BaseManager):
     obj_cls = ApplicationSettings
 
 
+class BroadcastMessage(GitlabObject):
+    _url = '/broadcast_messages'
+    requiredCreateAttrs = ['message']
+    optionalCreateAttrs = ['starts_at', 'ends_at', 'color', 'font']
+    requiredUpdateAttrs = []
+    optionalUpdateAttrs = ['message', 'starts_at', 'ends_at', 'color', 'font']
+
+
+class BroadcastMessageManager(BaseManager):
+    obj_cls = BroadcastMessage
+
+
 class Key(GitlabObject):
     _url = '/deploy_keys'
     canGet = 'from_list'
@@ -723,6 +787,54 @@ class KeyManager(BaseManager):
     obj_cls = Key
 
 
+class NotificationSettings(GitlabObject):
+    _url = '/notification_settings'
+    _id_in_update_url = False
+    optionalUpdateAttrs = ['level',
+                           'notification_email',
+                           'new_note',
+                           'new_issue',
+                           'reopen_issue',
+                           'close_issue',
+                           'reassign_issue',
+                           'new_merge_request',
+                           'reopen_merge_request',
+                           'close_merge_request',
+                           'reassign_merge_request',
+                           'merge_merge_request']
+    canList = False
+    canCreate = False
+    canDelete = False
+
+
+class NotificationSettingsManager(BaseManager):
+    obj_cls = NotificationSettings
+
+
+class Gitignore(GitlabObject):
+    _url = '/templates/gitignores'
+    canDelete = False
+    canUpdate = False
+    canCreate = False
+    idAttr = 'name'
+
+
+class GitignoreManager(BaseManager):
+    obj_cls = Gitignore
+
+
+class Gitlabciyml(GitlabObject):
+    _url = '/templates/gitlab_ci_ymls'
+    canDelete = False
+    canUpdate = False
+    canCreate = False
+    idAttr = 'name'
+
+
+class GitlabciymlManager(BaseManager):
+    obj_cls = Gitlabciyml
+
+
 class GroupIssue(GitlabObject):
     _url = '/groups/%(group_id)s/issues'
     canGet = 'from_list'
@@ -754,6 +866,15 @@ class GroupMemberManager(BaseManager):
     obj_cls = GroupMember
 
 
+class GroupNotificationSettings(NotificationSettings):
+    _url = '/groups/%(group_id)s/notification_settings'
+    requiredUrlAttrs = ['group_id']
+
+
+class GroupNotificationSettingsManager(BaseManager):
+    obj_cls = GroupNotificationSettings
+
+
 class GroupProject(GitlabObject):
     _url = '/groups/%(group_id)s/projects'
     canGet = 'from_list'
@@ -803,12 +924,14 @@ class Group(GitlabObject):
     optionalCreateAttrs = ['description', 'visibility_level']
     optionalUpdateAttrs = ['name', 'path', 'description', 'visibility_level']
     shortPrintAttr = 'name'
-    managers = [
+    managers = (
         ('accessrequests', GroupAccessRequestManager, [('group_id', 'id')]),
         ('members', GroupMemberManager, [('group_id', 'id')]),
+        ('notificationsettings', GroupNotificationSettingsManager,
+         [('group_id', 'id')]),
         ('projects', GroupProjectManager, [('group_id', 'id')]),
-        ('issues', GroupIssueManager, [('group_id', 'id')])
-    ]
+        ('issues', GroupIssueManager, [('group_id', 'id')]),
+    )
 
     GUEST_ACCESS = gitlab.GUEST_ACCESS
     REPORTER_ACCESS = gitlab.REPORTER_ACCESS
@@ -908,6 +1031,36 @@ class NamespaceManager(BaseManager):
     obj_cls = Namespace
 
 
+class ProjectBoardList(GitlabObject):
+    _url = '/projects/%(project_id)s/boards/%(board_id)s/lists'
+    requiredUrlAttrs = ['project_id', 'board_id']
+    _constructorTypes = {'label': 'ProjectLabel'}
+    requiredCreateAttrs = ['label_id']
+    requiredUpdateAttrs = ['position']
+
+
+class ProjectBoardListManager(BaseManager):
+    obj_cls = ProjectBoardList
+
+
+class ProjectBoard(GitlabObject):
+    _url = '/projects/%(project_id)s/boards'
+    requiredUrlAttrs = ['project_id']
+    _constructorTypes = {'labels': 'ProjectBoardList'}
+    canGet = 'from_list'
+    canUpdate = False
+    canCreate = False
+    canDelete = False
+    managers = (
+        ('lists', ProjectBoardListManager,
+            [('project_id', 'project_id'), ('board_id', 'id')]),
+    )
+
+
+class ProjectBoardManager(BaseManager):
+    obj_cls = ProjectBoard
+
+
 class ProjectBranch(GitlabObject):
     _url = '/projects/%(project_id)s/repository/branches'
     _constructorTypes = {'author': 'User', "committer": "User"}
@@ -965,7 +1118,7 @@ class ProjectBuild(GitlabObject):
         """Trigger a build explicitly."""
         url = '/projects/%s/builds/%s/play' % (self.project_id, self.id)
         r = self.gitlab._raw_post(url)
-        raise_error_from_response(r, GitlabBuildPlayError, 201)
+        raise_error_from_response(r, GitlabBuildPlayError)
 
     def erase(self, **kwargs):
         """Erase the build (remove build artifacts and trace)."""
@@ -1074,10 +1227,12 @@ class ProjectCommit(GitlabObject):
     canCreate = False
     requiredUrlAttrs = ['project_id']
     shortPrintAttr = 'title'
-    managers = [('comments', ProjectCommitCommentManager,
-                 [('project_id', 'project_id'), ('commit_id', 'id')]),
-                ('statuses', ProjectCommitStatusManager,
-                 [('project_id', 'project_id'), ('commit_id', 'id')])]
+    managers = (
+        ('comments', ProjectCommitCommentManager,
+            [('project_id', 'project_id'), ('commit_id', 'id')]),
+        ('statuses', ProjectCommitStatusManager,
+            [('project_id', 'project_id'), ('commit_id', 'id')]),
+    )
 
     def diff(self, **kwargs):
         """Generate the commit diff."""
@@ -1155,6 +1310,19 @@ class ProjectKey(GitlabObject):
     requiredUrlAttrs = ['project_id']
     requiredCreateAttrs = ['title', 'key']
 
+    def enable(self):
+        """Enable a deploy key for a project."""
+        url = '/projects/%s/deploy_keys/%s/enable' % (self.project_id, self.id)
+        r = self.gitlab._raw_post(url)
+        raise_error_from_response(r, GitlabProjectDeployKeyError, 201)
+
+    def disable(self):
+        """Disable a deploy key for a project."""
+        url = '/projects/%s/deploy_keys/%s/disable' % (self.project_id,
+                                                       self.id)
+        r = self.gitlab._raw_delete(url)
+        raise_error_from_response(r, GitlabProjectDeployKeyError, 200)
+
 
 class ProjectKeyManager(BaseManager):
     obj_cls = ProjectKey
@@ -1194,7 +1362,7 @@ class ProjectHook(GitlabObject):
     requiredCreateAttrs = ['url']
     optionalCreateAttrs = ['push_events', 'issues_events', 'note_events',
                            'merge_requests_events', 'tag_push_events',
-                           'build_events', 'enable_ssl_verification']
+                           'build_events', 'enable_ssl_verification', 'token']
     shortPrintAttr = 'url'
 
 
@@ -1208,6 +1376,7 @@ class ProjectIssueNote(GitlabObject):
     canDelete = False
     requiredUrlAttrs = ['project_id', 'issue_id']
     requiredCreateAttrs = ['body']
+    optionalCreateAttrs = ['created_at']
 
 
 class ProjectIssueNoteManager(BaseManager):
@@ -1226,10 +1395,12 @@ class ProjectIssue(GitlabObject):
                            'labels', 'created_at']
     optionalUpdateAttrs = ['title', 'description', 'assignee_id',
                            'milestone_id', 'labels', 'created_at',
-                           'state_event']
+                           'updated_at', 'state_event']
     shortPrintAttr = 'title'
-    managers = [('notes', ProjectIssueNoteManager,
-                 [('project_id', 'project_id'), ('issue_id', 'id')])]
+    managers = (
+        ('notes', ProjectIssueNoteManager,
+            [('project_id', 'project_id'), ('issue_id', 'id')]),
+    )
 
     def _data_for_gitlab(self, extra_parameters={}, update=False,
                          as_json=True):
@@ -1328,6 +1499,15 @@ class ProjectNoteManager(BaseManager):
     obj_cls = ProjectNote
 
 
+class ProjectNotificationSettings(NotificationSettings):
+    _url = '/projects/%(project_id)s/notification_settings'
+    requiredUrlAttrs = ['project_id']
+
+
+class ProjectNotificationSettingsManager(BaseManager):
+    obj_cls = ProjectNotificationSettings
+
+
 class ProjectTagRelease(GitlabObject):
     _url = '/projects/%(project_id)s/repository/tags/%(tag_name)/release'
     canDelete = False
@@ -1378,6 +1558,19 @@ class ProjectTagManager(BaseManager):
     obj_cls = ProjectTag
 
 
+class ProjectMergeRequestDiff(GitlabObject):
+    _url = ('/projects/%(project_id)s/merge_requests/'
+            '%(merge_request_id)s/versions')
+    canCreate = False
+    canUpdate = False
+    canDelete = False
+    requiredUrlAttrs = ['project_id', 'merge_request_id']
+
+
+class ProjectMergeRequestDiffManager(BaseManager):
+    obj_cls = ProjectMergeRequestDiff
+
+
 class ProjectMergeRequestNote(GitlabObject):
     _url = '/projects/%(project_id)s/merge_requests/%(merge_request_id)s/notes'
     _constructorTypes = {'author': 'User'}
@@ -1402,8 +1595,12 @@ class ProjectMergeRequest(GitlabObject):
                            'milestone_id']
     optionalListAttrs = ['iid', 'state', 'order_by', 'sort']
 
-    managers = [('notes', ProjectMergeRequestNoteManager,
-                 [('project_id', 'project_id'), ('merge_request_id', 'id')])]
+    managers = (
+        ('notes', ProjectMergeRequestNoteManager,
+            [('project_id', 'project_id'), ('merge_request_id', 'id')]),
+        ('diffs', ProjectMergeRequestDiffManager,
+            [('project_id', 'project_id'), ('merge_request_id', 'id')]),
+    )
 
     def _data_for_gitlab(self, extra_parameters={}, update=False,
                          as_json=True):
@@ -1502,7 +1699,7 @@ class ProjectMergeRequest(GitlabObject):
             GitlabConnectionError: If the server cannot be reached.
             GitlabListError: If the server fails to perform the request.
         """
-        url = ('/projects/%s/merge_requests/%s/commits' %
+        url = ('/projects/%s/merge_requests/%s/changes' %
                (self.project_id, self.id))
         r = self.gitlab._raw_get(url, **kwargs)
         raise_error_from_response(r, GitlabListError)
@@ -1535,9 +1732,9 @@ class ProjectMergeRequest(GitlabObject):
         if merge_commit_message:
             data['merge_commit_message'] = merge_commit_message
         if should_remove_source_branch:
-            data['should_remove_source_branch'] = 'should_remove_source_branch'
+            data['should_remove_source_branch'] = True
         if merged_when_build_succeeds:
-            data['merged_when_build_succeeds'] = 'merged_when_build_succeeds'
+            data['merged_when_build_succeeds'] = True
 
         r = self.gitlab._raw_put(url, data=data, **kwargs)
         errors = {401: GitlabMRForbiddenError,
@@ -1711,13 +1908,10 @@ class ProjectSnippet(GitlabObject):
     optionalCreateAttrs = ['lifetime', 'visibility_level']
     optionalUpdateAttrs = ['title', 'file_name', 'code', 'visibility_level']
     shortPrintAttr = 'title'
-    managers = [('notes', ProjectSnippetNoteManager,
-                 [('project_id', 'project_id'), ('snippet_id', 'id')])]
-
-    def Content(self, **kwargs):
-        warnings.warn("`Content` is deprecated, use `content` instead",
-                      DeprecationWarning)
-        return self.content()
+    managers = (
+        ('notes', ProjectSnippetNoteManager,
+            [('project_id', 'project_id'), ('snippet_id', 'id')]),
+    )
 
     def content(self, streamed=False, action=None, chunk_size=1024, **kwargs):
         """Return the raw content of a snippet.
@@ -1905,16 +2099,14 @@ class Project(GitlabObject):
                            'public_builds',
                            'only_allow_merge_if_build_succeeds']
     shortPrintAttr = 'path'
-    managers = [
+    managers = (
         ('accessrequests', ProjectAccessRequestManager,
          [('project_id', 'id')]),
+        ('boards', ProjectBoardManager, [('project_id', 'id')]),
+        ('board_lists', ProjectBoardListManager, [('project_id', 'id')]),
         ('branches', ProjectBranchManager, [('project_id', 'id')]),
         ('builds', ProjectBuildManager, [('project_id', 'id')]),
         ('commits', ProjectCommitManager, [('project_id', 'id')]),
-        ('commit_comments', ProjectCommitCommentManager,
-         [('project_id', 'id')]),
-        ('commit_statuses', ProjectCommitStatusManager,
-         [('project_id', 'id')]),
         ('deployments', ProjectDeploymentManager, [('project_id', 'id')]),
         ('environments', ProjectEnvironmentManager, [('project_id', 'id')]),
         ('events', ProjectEventManager, [('project_id', 'id')]),
@@ -1928,23 +2120,20 @@ class Project(GitlabObject):
         ('mergerequests', ProjectMergeRequestManager, [('project_id', 'id')]),
         ('milestones', ProjectMilestoneManager, [('project_id', 'id')]),
         ('notes', ProjectNoteManager, [('project_id', 'id')]),
+        ('notificationsettings', ProjectNotificationSettingsManager,
+         [('project_id', 'id')]),
         ('pipelines', ProjectPipelineManager, [('project_id', 'id')]),
         ('services', ProjectServiceManager, [('project_id', 'id')]),
         ('snippets', ProjectSnippetManager, [('project_id', 'id')]),
         ('tags', ProjectTagManager, [('project_id', 'id')]),
         ('triggers', ProjectTriggerManager, [('project_id', 'id')]),
         ('variables', ProjectVariableManager, [('project_id', 'id')]),
-    ]
+    )
 
     VISIBILITY_PRIVATE = gitlab.VISIBILITY_PRIVATE
     VISIBILITY_INTERNAL = gitlab.VISIBILITY_INTERNAL
     VISIBILITY_PUBLIC = gitlab.VISIBILITY_PUBLIC
 
-    def tree(self, path='', ref_name='', **kwargs):
-        warnings.warn("`tree` is deprecated, use `repository_tree` instead",
-                      DeprecationWarning)
-        return self.repository_tree(path, ref_name, **kwargs)
-
     def repository_tree(self, path='', ref_name='', **kwargs):
         """Return a list of files in the repository.
 
@@ -1971,11 +2160,6 @@ class Project(GitlabObject):
         raise_error_from_response(r, GitlabGetError)
         return r.json()
 
-    def blob(self, sha, filepath, **kwargs):
-        warnings.warn("`blob` is deprecated, use `repository_blob` instead",
-                      DeprecationWarning)
-        return self.repository_blob(sha, filepath, **kwargs)
-
     def repository_blob(self, sha, filepath, streamed=False, action=None,
                         chunk_size=1024, **kwargs):
         """Return the content of a file for a commit.
@@ -2063,12 +2247,6 @@ class Project(GitlabObject):
         raise_error_from_response(r, GitlabListError)
         return r.json()
 
-    def archive(self, sha=None, **kwargs):
-        warnings.warn("`archive` is deprecated, "
-                      "use `repository_archive` instead",
-                      DeprecationWarning)
-        return self.repository_archive(sha, **kwargs)
-
     def repository_archive(self, sha=None, streamed=False, action=None,
                            chunk_size=1024, **kwargs):
         """Return a tarball of the repository.
@@ -2096,49 +2274,6 @@ class Project(GitlabObject):
         raise_error_from_response(r, GitlabGetError)
         return utils.response_content(r, streamed, action, chunk_size)
 
-    def create_file(self, path, branch, content, message, **kwargs):
-        """Creates file in project repository
-
-        Args:
-            path (str): Full path to new file.
-            branch (str): The name of branch.
-            content (str): Content of the file.
-            message (str): Commit message.
-            **kwargs: Arbitrary keyword arguments.
-
-        Raises:
-            GitlabConnectionError: If the server cannot be reached.
-            GitlabCreateError: If the server fails to perform the request.
-        """
-        warnings.warn("`create_file` is deprecated, "
-                      "use `files.create()` instead",
-                      DeprecationWarning)
-        url = "/projects/%s/repository/files" % self.id
-        url += ("?file_path=%s&branch_name=%s&content=%s&commit_message=%s" %
-                (path, branch, content, message))
-        r = self.gitlab._raw_post(url, data=None, content_type=None, **kwargs)
-        raise_error_from_response(r, GitlabCreateError, 201)
-
-    def update_file(self, path, branch, content, message, **kwargs):
-        warnings.warn("`update_file` is deprecated, "
-                      "use `files.update()` instead",
-                      DeprecationWarning)
-        url = "/projects/%s/repository/files" % self.id
-        url += ("?file_path=%s&branch_name=%s&content=%s&commit_message=%s" %
-                (path, branch, content, message))
-        r = self.gitlab._raw_put(url, data=None, content_type=None, **kwargs)
-        raise_error_from_response(r, GitlabUpdateError)
-
-    def delete_file(self, path, branch, message, **kwargs):
-        warnings.warn("`delete_file` is deprecated, "
-                      "use `files.delete()` instead",
-                      DeprecationWarning)
-        url = "/projects/%s/repository/files" % self.id
-        url += ("?file_path=%s&branch_name=%s&commit_message=%s" %
-                (path, branch, message))
-        r = self.gitlab._raw_delete(url, **kwargs)
-        raise_error_from_response(r, GitlabDeleteError)
-
     def create_fork_relation(self, forked_from_id):
         """Create a forked from/to relation between existing projects.
 
@@ -2194,7 +2329,7 @@ class Project(GitlabObject):
         raise_error_from_response(r, GitlabDeleteError, [200, 304])
         return Project(self.gitlab, r.json()) if r.status_code == 200 else self
 
-    def archive_(self, **kwargs):
+    def archive(self, **kwargs):
         """Archive a project.
 
         Returns:
@@ -2209,7 +2344,12 @@ class Project(GitlabObject):
         raise_error_from_response(r, GitlabCreateError, 201)
         return Project(self.gitlab, r.json()) if r.status_code == 201 else self
 
-    def unarchive_(self, **kwargs):
+    def archive_(self, **kwargs):
+        warnings.warn("`archive_()` is deprecated, use `archive()` instead",
+                      DeprecationWarning)
+        return self.archive(**kwargs)
+
+    def unarchive(self, **kwargs):
         """Unarchive a project.
 
         Returns:
@@ -2224,6 +2364,12 @@ class Project(GitlabObject):
         raise_error_from_response(r, GitlabCreateError, 201)
         return Project(self.gitlab, r.json()) if r.status_code == 201 else self
 
+    def unarchive_(self, **kwargs):
+        warnings.warn("`unarchive_()` is deprecated, "
+                      "use `unarchive()` instead",
+                      DeprecationWarning)
+        return self.unarchive(**kwargs)
+
     def share(self, group_id, group_access, **kwargs):
         """Share the project with a group.
 
@@ -2240,6 +2386,27 @@ class Project(GitlabObject):
         r = self.gitlab._raw_post(url, data=data, **kwargs)
         raise_error_from_response(r, GitlabCreateError, 201)
 
+    def trigger_build(self, ref, token, variables={}, **kwargs):
+        """Trigger a CI build.
+
+        See https://gitlab.com/help/ci/triggers/README.md#trigger-a-build
+
+        Args:
+            ref (str): Commit to build; can be a commit SHA, a branch name, ...
+            token (str): The trigger token
+            variables (dict): Variables passed to the build script
+
+        Raises:
+            GitlabConnectionError: If the server cannot be reached.
+            GitlabCreateError: If the server fails to perform the request.
+        """
+        url = "/projects/%s/trigger/builds" % self.id
+        form = {r'variables[%s]' % k: v for k, v in six.iteritems(variables)}
+        data = {'ref': ref, 'token': token}
+        data.update(form)
+        r = self.gitlab._raw_post(url, data=data, **kwargs)
+        raise_error_from_response(r, GitlabCreateError, 201)
+
 
 class Runner(GitlabObject):
     _url = '/runners'
@@ -2305,22 +2472,6 @@ class TodoManager(BaseManager):
         return int(r.text)
 
 
-class UserProject(GitlabObject):
-    _url = '/projects/user/%(user_id)s'
-    _constructorTypes = {'owner': 'User', 'namespace': 'Group'}
-    canUpdate = False
-    canDelete = False
-    canList = False
-    canGet = False
-    requiredUrlAttrs = ['user_id']
-    requiredCreateAttrs = ['name']
-    optionalCreateAttrs = ['default_branch', 'issues_enabled', 'wall_enabled',
-                           'merge_requests_enabled', 'wiki_enabled',
-                           'snippets_enabled', 'public', 'visibility_level',
-                           'description', 'builds_enabled', 'public_builds',
-                           'import_url', 'only_allow_merge_if_build_succeeds']
-
-
 class ProjectManager(BaseManager):
     obj_cls = Project
 
@@ -2385,10 +2536,6 @@ class ProjectManager(BaseManager):
         return self.gitlab._raw_list("/projects/starred", Project, **kwargs)
 
 
-class UserProjectManager(BaseManager):
-    obj_cls = UserProject
-
-
 class TeamMemberManager(BaseManager):
     obj_cls = TeamMember
 
@@ -2411,10 +2558,10 @@ class Team(GitlabObject):
     shortPrintAttr = 'name'
     requiredCreateAttrs = ['name', 'path']
     canUpdate = False
-    managers = [
+    managers = (
         ('members', TeamMemberManager, [('team_id', 'id')]),
-        ('projects', TeamProjectManager, [('team_id', 'id')])
-    ]
+        ('projects', TeamProjectManager, [('team_id', 'id')]),
+    )
 
 
 class TeamManager(BaseManager):
